/*

  BorderLayout Customizer

  An example of a constraints based layout manager customizer
  for JForge.

  Copyright (c)1998, Cameron Newham.
  This file is part of the JForge archive and is subject to the
  same copyright, warranty and usage conditions as layed out in
  the legal statement that comes with the JForge archive.
  No Warranty. Use at your own risk.
*/

package java.awt;

import BuilderLayoutCustomizer;
import java.awt.*;
import java.awt.event.*;
import java.beans.*;
import com.sun.java.swing.*;
import java.util.Vector;

public class BorderLayoutCustomizer extends BorderLayoutCustomizerGui
                                    implements Customizer,
                                               BuilderLayoutCustomizer,
                                               KeyListener,
                                               PropertyChangeListener,
                                               ActionListener
{
  private PropertyChangeListener listener;
  private BorderLayout           layout;
  private Container              container;
  private ConstraintPanel[]      cp;
  private Component[]            comps;

  public BorderLayoutCustomizer()
  {
    super();
    txtHGap.addKeyListener(this);
    txtVGap.addKeyListener(this);
    // no need to register ourselves as a property change listener - it is
    // done implicitely by whoever creates us (and if they want to).
  }


  public void addPropertyChangeListener(PropertyChangeListener listener)
  {
    this.listener = listener;  // only one needed
  }


  public void removePropertyChangeListener(PropertyChangeListener listener)
  {
    this.listener = null;
  }


  public void setObject(Object target)
  {
    container = (Container)target;
    layout = (BorderLayout)container.getLayout();
    changeCustomizer();
  }


  private void changeCustomizer()
  {
    // obtain the latest info from the container and layout manager
    // and display it in the customizer GUI

    txtHGap.setText(""+layout.getHgap());
    txtVGap.setText(""+layout.getVgap());
    comps = container.getComponents();

    cp = new ConstraintPanel[comps.length];

    BLButtonGroup bg = new BLButtonGroup(5+comps.length);

    bg.addActionListener(this);

    for (int i = 0; i < comps.length; i++)
    {
      cp[i] = new ConstraintPanel(comps[i].getName());

      for (int j = 0; j < 5; j++)
      {
        bg.add(cp[i].compCons[j], i+5);  // add the button to the vertical
        bg.add(cp[i].compCons[j], j);    // and horizontal groups
      }

      if (layout.center == comps[i])
        cp[i].compCons[0].setSelected(true);
      else if (layout.north == comps[i])
        cp[i].compCons[1].setSelected(true);
      else if (layout.south == comps[i])
        cp[i].compCons[2].setSelected(true);
      else if (layout.east == comps[i])
        cp[i].compCons[3].setSelected(true);
      else if (layout.west == comps[i])
        cp[i].compCons[4].setSelected(true);

      pnlList.add(cp[i]);
    }
    pnlList.setSize(280,21*comps.length);
  }


  public String[] getJavaInitializationStrings(Container container)
  {
    BorderLayout layout = (BorderLayout)container.getLayout();
    Component[] comps = container.getComponents();
    String[] strings = new String[comps.length+4];
    String containerName = container.getName();
    String layoutName = containerName+"Layout";

    strings[0] = "BorderLayout "+layoutName+" = new BorderLayout();";
    strings[1] = containerName+".setLayout("+layoutName+");";
    strings[2] = layoutName+".setHgap("+layout.getHgap()+");";
    strings[3] = layoutName+".setVgap("+layout.getVgap()+");";

    for (int i = 0; i < comps.length; i++)
    {
      String posn = "CENTER";

      if (layout.center == comps[i])
        posn = "CENTER";
      else if (layout.north == comps[i])
        posn = "NORTH";
      else if (layout.south == comps[i])
        posn = "SOUTH";
      else if (layout.east == comps[i])
        posn = "EAST";
      else if (layout.west == comps[i])
        posn = "WEST";
      strings[i+4] = containerName+".add("+comps[i].getName()+
                                           ", BorderLayout."+posn+");";
    }
      
    return strings;
  }


  public void addLayout(Container container)
  {
    container.setLayout(new BorderLayout());
  }


  public void keyPressed(KeyEvent evt)
  {
    // nothing to do
  }


  public void keyReleased(KeyEvent evt)
  {
    customizerChanged();
  }


  public void keyTyped(KeyEvent evt)
  {
    // nothing to do
  }


  public void actionPerformed(ActionEvent evt)
  {
    customizerChanged();
  }


  private void customizerChanged()
  {
    // grab values from the Customizer and notify the listener of the change
    try
    {
      layout.setHgap(Integer.parseInt(txtHGap.getText()));
      layout.setVgap(Integer.parseInt(txtVGap.getText()));
    }
    catch (NumberFormatException nfe)
    {
      // Silently ignore...
    }

    for (int i = 0; i < cp.length; i++)
    {
      if (cp[i].compCons[0].isSelected())
      {
        container.remove(comps[i]);
        container.add(comps[i], BorderLayout.CENTER);
      }
      else if (cp[i].compCons[1].isSelected())
      {
        container.remove(comps[i]);
        container.add(comps[i], BorderLayout.NORTH);
      }
      else if (cp[i].compCons[2].isSelected())
      {
        container.remove(comps[i]);
        container.add(comps[i], BorderLayout.SOUTH);
      }
      else if (cp[i].compCons[3].isSelected())
      {
        container.remove(comps[i]);
        container.add(comps[i], BorderLayout.EAST);
      }
      else if (cp[i].compCons[4].isSelected())
      {
        container.remove(comps[i]);
        container.add(comps[i], BorderLayout.WEST);
      }
    }

    if (listener != null)
      listener.propertyChange(new PropertyChangeEvent(container,
                                                      null,
                                                      null,
                                                      null));
  }



  public void propertyChange(PropertyChangeEvent evt)
  {
    changeCustomizer();
  }


  private class ConstraintPanel extends JPanel
  {
    JLabel       compName = new JLabel();
    JRadioButton[] compCons = new JRadioButton[5];

    ConstraintPanel(String text)
    {
      super();
      compCons[0] = new JRadioButton("C");
      compCons[1] = new JRadioButton("N");
      compCons[2] = new JRadioButton("S");
      compCons[3] = new JRadioButton("E");
      compCons[4] = new JRadioButton("W");
      this.setSize(270,21);
      compName.setPreferredSize(new Dimension(75,21));
      compName.setText(text);
      compName.setToolTipText(text);
      this.setLayout(new FlowLayout());
      this.add(compName);
      for (int i = 0; i < compCons.length; i++)
      {
        this.add(compCons[i]);
      }
    }
  }


  private class BLButtonGroup implements ActionListener
  {

    // A Button Group that maintains several groups of buttons.
    // A button may be a member of one or more groups.

    Vector         buttons[];
    ActionListener listener;
    int            numGroups;

    BLButtonGroup(int numGroups)
    {
      this.numGroups = numGroups;
      buttons = new Vector[numGroups];
      for (int i = 0; i < numGroups; i++)
        buttons[i] = new Vector();
    }

    void add(AbstractButton button, int group)
    {
      buttons[group].addElement(button);
      button.addActionListener(this);
    }


    void addActionListener(ActionListener listener)
    {
      this.listener = listener;
    }


    public void actionPerformed(ActionEvent evt)
    {
      Object source = evt.getSource();


      // loop through the groups. if the target is in a particular
      // group, then deselect all buttons in the group (except the
      // target).

      for (int groupNum = 0; groupNum < numGroups; groupNum++)
        if (buttons[groupNum].contains(source))
          for (int i = 0; i < buttons[groupNum].size(); i++)
          {
            AbstractButton button = (AbstractButton)buttons[groupNum].
                                                                elementAt(i);
            if (button != source)
              button.setSelected(false);
          }

      if (listener != null)
      {
        listener.actionPerformed(evt);  // pass the event through - this is
                                        // important as selection code
                                        // (above) must happen before anyone
                                        // else is told about the action.
      }
    }
  }
}
