package pub.java.gui.builders;

import java.awt.Component;
import java.awt.Container;

public interface BuilderComponentBeanInfo
{
  /*--------------------------------------------------------------------
    Returns a new instance of the component. For simple components this
    could be: return new MyComponent();  For components that need to
    do special initialisation (e.g.: JTabbedPane automatically inserts
    a tab and a panel for convenience), return the component after
    doing initialisation.
  --------------------------------------------------------------------*/
  public Component newInstance();

  /*--------------------------------------------------------------------
    Returns a component to which layouts and other components can
    be added. For simple components this would be:  return component;
    For more complex components, e.g. JFrame, it could be:
    return component.getContentPane(); A return of null indicates
    that the component can't be added to. For components with multiple
    content places (e.g.: JTabbedPane) it should return the currently
    visible content.
  --------------------------------------------------------------------*/
  public Container getContent(Component component);

  /*--------------------------------------------------------------------
    Returns the children of this component (i.e.: those children
    that can be editted and are managed by this component). For
    simple components (containers) it could be: container.getComponents();
    For more complex components (e.g.: JTabbedPane) it should return
    all managed children. May return null.
  --------------------------------------------------------------------*/
  public Component[] getChildren(Component component);

  /*--------------------------------------------------------------------
    Returns the currently visible children of this component. For
    a simple component this could be: container.getComponents();
    For more complex components (e.g.: JTabbedPane) it should return
    the currently visible children (visible as in being potentially able
    to be seen on the screen - which includes components with
    isVisible() == false). This could be as simple as a call to:
    getContent().getComponents(); May return null.
  --------------------------------------------------------------------*/
  public Component[] getVisibleChildren(Component component);

  /*--------------------------------------------------------------------
    Returns an array of strings that would regenerate the component
    when compiled and run.  A return value of null indicates that
    JForge should use its own code generation code to write out the
    component.  If you provide this, you may use the JForge Code
    Generation API to generate code for child components and layouts.
  --------------------------------------------------------------------*/
  public String[] getJavaInitializationStrings(Component component);
}
