package pub.java.gui.builders;

/*
******************* update
  This is the Customizer Interface needed for building JForge-friendly
  Layout Manager Customizers.  In the case of simple layouts, JForge
  can introspect them and use them as-is.  However, if your layout
  manager;

    a) Is constraint based, or
    b) Doesn't provide a null argument constructor, or
    c) Doesn't get associated to a container via the setLayout()
       method

  then you'll need to use a customizer which implements this interface.

  The object passed into the setObject() method of the customizer will
  be a container for which the layout needs to be manipulated.

  This code is Copyright (c)1998, Cameron Newham.
*/

import java.awt.Container;
import java.beans.*;

public interface BuilderLayoutBeanInfo
{

  /*--------------------------------------------------------------------
    container - The container associated with the layout.

    Returns an array of strings. The strings should provide the correct
    source code lines needed to add the components in the container to
    the container using the layout manager associated with the container.
    The names of the container and components are given by the getName()
    method.  The name of the layout should be of the form <layout>Layout,
    which can be produced with:  "myLayout"+container.getName()
  --------------------------------------------------------------------*/
  public String[] getJavaInitializationStrings(Container container);


  /*--------------------------------------------------------------------
    container - the container for which a layout needs to be added.

    Layouts are always added via this method. For simple layouts, this
    would be something like:
      container.setLayout(new myLayout());
  --------------------------------------------------------------------*/
  public void addLayout(Container container);


  /*--------------------------------------------------------------------
    A layout manager customiser has the potential to be implicitly
    added to a listener list when it is created. This method should
    do whatever is needed when a property change occurs.  As the
    registration is done implicitly, there is no need to call an
    addPropertyChangeListener() in the class.
  --------------------------------------------------------------------*/
//*************
//  public void propertyChange(PropertyChangeEvent evt);
}

